@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("3.0.") or
-    raise "ruby lib version (3.0.1) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (3.0.2) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.0/arm-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
   CONFIG["MINOR"] = "0"
-  CONFIG["TEENY"] = "1"
-  CONFIG["PATCHLEVEL"] = "64"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.1-no_ri_docs-bootstrap/bin/install -c'
+  CONFIG["TEENY"] = "2"
+  CONFIG["PATCHLEVEL"] = "107"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.0.2-no_ri_docs-bootstrap/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)30"
@@ -44,7 +44,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=30' '--with-soname=ruby30' '--with-ruby-version=minor' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--enable-shared' '--program-suffix=30' '--with-soname=ruby30' '--with-ruby-version=minor' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -142,16 +142,14 @@ module RbConfig
   CONFIG["RMDIR"] = "rmdir"
   CONFIG["CP"] = "cp"
   CONFIG["RM"] = "rm -f"
-  CONFIG["EGREP"] = "/usr/bin/egrep"
-  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["PKG_CONFIG"] = "pkg-config"
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.1-no_ri_docs-bootstrap/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.1-no_ri_docs-bootstrap/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.1-no_ri_docs-bootstrap/bin/install -c -s -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.0.2-no_ri_docs-bootstrap/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.0.2-no_ri_docs-bootstrap/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.0.2-no_ri_docs-bootstrap/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
@@ -159,7 +157,7 @@ module RbConfig
   CONFIG["ASFLAGS"] = ""
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 10.0.1 \nTarget: armv7-unknown-openbsd6.9-gnueabi\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 11.1.0\nTarget: armv7-unknown-openbsd7.0-gnueabi\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["MJIT_CC"] = "/usr/bin/cc -fdeclspec"
   CONFIG["CSRCFLAG"] = ""
@@ -168,6 +166,8 @@ module RbConfig
   CONFIG["CPPOUTFILE"] = "-o conftest.i"
   CONFIG["GNU_LD"] = "no"
   CONFIG["GCC"] = "yes"
+  CONFIG["EGREP"] = "/usr/bin/egrep"
+  CONFIG["GREP"] = "/usr/bin/grep"
   CONFIG["CPP"] = "cc -E"
   CONFIG["CXXFLAGS"] = "-O2 -pipe"
   CONFIG["OBJEXT"] = "o"
@@ -181,21 +181,21 @@ module RbConfig
   CONFIG["NM"] = "nm"
   CONFIG["LD"] = "ld"
   CONFIG["CXX"] = "c++ -fdeclspec"
-  CONFIG["AS"] = "as"
+  CONFIG["AS"] = "gas"
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc -fdeclspec"
-  CONFIG["target_os"] = "openbsd6.9"
+  CONFIG["target_os"] = "openbsd7.0"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "arm"
-  CONFIG["target"] = "arm-unknown-openbsd6.9"
-  CONFIG["host_os"] = "openbsd6.9"
+  CONFIG["target"] = "arm-unknown-openbsd7.0"
+  CONFIG["host_os"] = "openbsd7.0"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "arm"
-  CONFIG["host"] = "arm-unknown-openbsd6.9"
-  CONFIG["build_os"] = "openbsd6.9"
+  CONFIG["host"] = "arm-unknown-openbsd7.0"
+  CONFIG["build_os"] = "openbsd7.0"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "arm"
-  CONFIG["build"] = "arm-unknown-openbsd6.9"
+  CONFIG["build"] = "arm-unknown-openbsd7.0"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
