@@ -1,7 +1,5 @@
 # frozen_string_literal: true
 
-require "rubygems/util"
-
 module Gem::BundlerVersionFinder
   def self.bundler_version
     version, _ = bundler_version_with_reason
@@ -44,9 +42,9 @@ To install the missing version, run `gem install bundler:#{vr.first}`
   def self.filter!(specs)
     return unless bundler_version = self.bundler_version
 
-    specs.reject! { |spec| spec.version.segments.first != bundler_version.segments.first }
+    specs.reject! {|spec| spec.version.segments.first != bundler_version.segments.first }
 
-    exact_match_index = specs.find_index { |spec| spec.version == bundler_version }
+    exact_match_index = specs.find_index {|spec| spec.version == bundler_version }
     return unless exact_match_index
 
     specs.unshift(specs.delete_at(exact_match_index))
@@ -82,12 +80,19 @@ To install the missing version, run `gem install bundler:#{vr.first}`
   def self.lockfile_contents
     gemfile = ENV["BUNDLE_GEMFILE"]
     gemfile = nil if gemfile && gemfile.empty?
-    Gem::Util.traverse_parents Dir.pwd do |directory|
-      next unless gemfile = Gem::GEM_DEP_FILES.find { |f| File.file?(f.tap(&Gem::UNTAINT)) }
 
-      gemfile = File.join directory, gemfile
-      break
-    end unless gemfile
+    unless gemfile
+      begin
+        Gem::Util.traverse_parents(Dir.pwd) do |directory|
+          next unless gemfile = Gem::GEM_DEP_FILES.find {|f| File.file?(f.tap(&Gem::UNTAINT)) }
+
+          gemfile = File.join directory, gemfile
+          break
+        end
+      rescue Errno::ENOENT
+        return
+      end
+    end
 
     return unless gemfile
 
