@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 ##
 # The Version class processes string versions into comparable
 # values. A version string should normally be a series of numbers
@@ -107,7 +108,7 @@
 # Client A needs a stack with basic push/pop capability.  They write to the
 # original interface (no <tt>top</tt>), so their version constraint looks like:
 #
-#   gem 'stack', '~> 0.0'
+#   gem 'stack', '>= 0.0'
 #
 # Essentially, any version is OK with Client A.  An incompatible change to
 # the library will cause them grief, but they are willing to take the chance
@@ -143,14 +144,18 @@
 #   "~> 3.0.0"    3.0.0 ... 3.1
 #   "~> 3.5"      3.5   ... 4.0
 #   "~> 3.5.0"    3.5.0 ... 3.6
+#   "~> 3"        3.0   ... 4.0
+#
+# For the last example, single-digit versions are automatically extended with
+# a zero to give a sensible result.
 
 class Gem::Version
-  autoload :Requirement, 'rubygems/requirement'
+  autoload :Requirement, File.expand_path('requirement', __dir__)
 
   include Comparable
 
-  VERSION_PATTERN = '[0-9]+(?>\.[0-9a-zA-Z]+)*(-[0-9A-Za-z-]+(\.[0-9A-Za-z-]+)*)?' # :nodoc:
-  ANCHORED_VERSION_PATTERN = /\A\s*(#{VERSION_PATTERN})?\s*\z/ # :nodoc:
+  VERSION_PATTERN = '[0-9]+(?>\.[0-9a-zA-Z]+)*(-[0-9A-Za-z-]+(\.[0-9A-Za-z-]+)*)?'.freeze # :nodoc:
+  ANCHORED_VERSION_PATTERN = /\A\s*(#{VERSION_PATTERN})?\s*\z/.freeze # :nodoc:
 
   ##
   # A string representation of this Version.
@@ -164,8 +169,12 @@ class Gem::Version
   ##
   # True if the +version+ string matches RubyGems' requirements.
 
-  def self.correct? version
-    version.to_s =~ ANCHORED_VERSION_PATTERN
+  def self.correct?(version)
+    unless Gem::Deprecate.skip
+      warn "nil versions are discouraged and will be deprecated in Rubygems 4" if version.nil?
+    end
+
+    !!(version.to_s =~ ANCHORED_VERSION_PATTERN)
   end
 
   ##
@@ -176,10 +185,10 @@ class Gem::Version
   #   ver2 = Version.create(ver1)       # -> (ver1)
   #   ver3 = Version.create(nil)        # -> nil
 
-  def self.create input
-    if self === input then # check yourself before you wreck yourself
+  def self.create(input)
+    if self === input # check yourself before you wreck yourself
       input
-    elsif input.nil? then
+    elsif input.nil?
       nil
     else
       new input
@@ -187,9 +196,11 @@ class Gem::Version
   end
 
   @@all = {}
+  @@bump = {}
+  @@release = {}
 
-  def self.new version # :nodoc:
-    return super unless Gem::VERSION == self.class
+  def self.new(version) # :nodoc:
+    return super unless Gem::Version == self
 
     @@all[version] ||= super
   end
@@ -198,9 +209,13 @@ class Gem::Version
   # Constructs a Version from the +version+ string.  A version string is a
   # series of digits or ASCII letters separated by dots.
 
-  def initialize version
-    raise ArgumentError, "Malformed version number string #{version}" unless
-      self.class.correct?(version)
+  def initialize(version)
+    unless self.class.correct?(version)
+      raise ArgumentError, "Malformed version number string #{version}"
+    end
+
+    # If version is an empty string convert it to 0
+    version = 0 if version.is_a?(String) && version =~ /\A\s*\Z/
 
     @version = version.to_s.strip.gsub("-",".pre.")
     @segments = nil
@@ -213,27 +228,29 @@ class Gem::Version
   # Pre-release (alpha) parts, e.g, 5.3.1.b.2 => 5.4, are ignored.
 
   def bump
-    segments = self.segments.dup
-    segments.pop while segments.any? { |s| String === s }
-    segments.pop if segments.size > 1
+    @@bump[self] ||= begin
+                       segments = self.segments
+                       segments.pop while segments.any? {|s| String === s }
+                       segments.pop if segments.size > 1
 
-    segments[-1] = segments[-1].succ
-    self.class.new segments.join(".")
+                       segments[-1] = segments[-1].succ
+                       self.class.new segments.join(".")
+                     end
   end
 
   ##
   # A Version is only eql? to another version if it's specified to the
   # same precision. Version "1.0" is not the same as version "1".
 
-  def eql? other
-    self.class === other and @version == other.version
+  def eql?(other)
+    self.class === other and @version == other._version
   end
 
   def hash # :nodoc:
-    @hash ||= segments.hash
+    canonical_segments.hash
   end
 
-  def init_with coder # :nodoc:
+  def init_with(coder) # :nodoc:
     yaml_initialize coder.tag, coder.map
   end
 
@@ -253,7 +270,7 @@ class Gem::Version
   # Load custom marshal format. It's a string for backwards (RubyGems
   # 1.3.5 and earlier) compatibility.
 
-  def marshal_load array
+  def marshal_load(array)
     initialize array[0]
   end
 
@@ -267,7 +284,7 @@ class Gem::Version
     ["@version"]
   end
 
-  def encode_with coder # :nodoc:
+  def encode_with(coder) # :nodoc:
     coder.add 'version', @version
   end
 
@@ -275,10 +292,13 @@ class Gem::Version
   # A version is considered a prerelease if it contains a letter.
 
   def prerelease?
-    @prerelease ||= !!(@version =~ /[a-zA-Z]/)
+    unless instance_variable_defined? :@prerelease
+      @prerelease = !!(@version =~ /[a-zA-Z]/)
+    end
+    @prerelease
   end
 
-  def pretty_print q # :nodoc:
+  def pretty_print(q) # :nodoc:
     q.text "Gem::Version.new(#{version.inspect})"
   end
 
@@ -287,34 +307,32 @@ class Gem::Version
   # Non-prerelease versions return themselves.
 
   def release
-    return self unless prerelease?
-
-    segments = self.segments.dup
-    segments.pop while segments.any? { |s| String === s }
-    self.class.new segments.join('.')
+    @@release[self] ||= if prerelease?
+                          segments = self.segments
+                          segments.pop while segments.any? {|s| String === s }
+                          self.class.new segments.join('.')
+                        else
+                          self
+                        end
   end
 
   def segments # :nodoc:
-
-    # segments is lazy so it can pick up version values that come from
-    # old marshaled versions, which don't go through marshal_load.
-
-    @segments ||= @version.scan(/[0-9]+|[a-z]+/i).map do |s|
-      /^\d+$/ =~ s ? s.to_i : s
-    end
+    _segments.dup
   end
 
   ##
   # A recommended version for use with a ~> Requirement.
 
   def approximate_recommendation
-    segments = self.segments.dup
+    segments = self.segments
 
-    segments.pop    while segments.any? { |s| String === s }
+    segments.pop    while segments.any? {|s| String === s }
     segments.pop    while segments.size > 2
     segments.push 0 while segments.size < 2
 
-    "~> #{segments.join(".")}"
+    recommendation = "~> #{segments.join(".")}"
+    recommendation += ".a" if prerelease?
+    recommendation
   end
 
   ##
@@ -323,12 +341,12 @@ class Gem::Version
   # one. Attempts to compare to something that's not a
   # <tt>Gem::Version</tt> return +nil+.
 
-  def <=> other
+  def <=>(other)
     return unless Gem::Version === other
-    return 0 if @version == other.version
+    return 0 if @version == other._version || canonical_segments == other.canonical_segments
 
-    lhsegments = segments
-    rhsegments = other.segments
+    lhsegments = canonical_segments
+    rhsegments = other.canonical_segments
 
     lhsize = lhsegments.size
     rhsize = rhsegments.size
@@ -349,4 +367,40 @@ class Gem::Version
 
     return 0
   end
+
+  def canonical_segments
+    @canonical_segments ||=
+      _split_segments.map! do |segments|
+        segments.reverse_each.drop_while {|s| s == 0 }.reverse
+      end.reduce(&:concat)
+  end
+
+  def freeze
+    prerelease?
+    canonical_segments
+    super
+  end
+
+  protected
+
+  def _version
+    @version
+  end
+
+  def _segments
+    # segments is lazy so it can pick up version values that come from
+    # old marshaled versions, which don't go through marshal_load.
+    # since this version object is cached in @@all, its @segments should be frozen
+
+    @segments ||= @version.scan(/[0-9]+|[a-z]+/i).map do |s|
+      /^\d+$/ =~ s ? s.to_i : s
+    end.freeze
+  end
+
+  def _split_segments
+    string_start = _segments.index {|s| s.is_a?(String) }
+    string_segments = segments
+    numeric_segments = string_segments.slice!(0, string_start || string_segments.size)
+    return numeric_segments, string_segments
+  end
 end
